From b202131097e2774a8e96f0d0e72c07c693977008 Mon Sep 17 00:00:00 2001
From: Ruslan Bekenev <furyinbox@gmail.com>
Date: Wed, 10 Apr 2024 12:58:07 +1000
Subject: [PATCH] Test dired delete functionality

Add test for https://debbugs.gnu.org/69117
The issue described in the bug is that a file cannot be trashed
if a file with the same name already exists in the trash bin.
The test checks this functionality and ensures that the dired delete
functionality works even in case trash bin already has the file with
the same name.
---
 test/lisp/dired-tests.el | 46 ++++++++++++++++++++++++++++++++++++++++
 1 file changed, 46 insertions(+)

diff --git a/test/lisp/dired-tests.el b/test/lisp/dired-tests.el
index 651b77500a1..d6773ac8a56 100644
--- a/test/lisp/dired-tests.el
+++ b/test/lisp/dired-tests.el
@@ -609,5 +609,51 @@ dired-test-directory-files-and-attributes
       (let ((default-directory test-dir-other))
         (files-tests--insert-directory-shows-given-free test-dir)))))
 
+(ert-deftest dired-test-bug69117 ()
+  "Test for https://debbugs.gnu.org/69117."
+  (let* ((delete-by-moving-to-trash t)
+         (dired-recursive-deletes 'always)
+         (dired-deletion-confirmer (lambda (_) t))
+         (testdir (expand-file-name "bug69117" default-directory))
+         (trash-directory (expand-file-name "trashdir" default-directory))
+         (dir-to-delete "dir-to-delete"))
+    (fmakunbound 'system-move-file-to-trash) ;; make sure file aren't moved to system trash
+    (unwind-protect
+        (progn
+          (mapc (lambda (d)
+                    (when (file-directory-p d)
+                      (delete-directory d t)))
+                  (list trash-directory testdir))
+          (make-directory trash-directory)
+          (make-directory testdir)
+          (when (file-directory-p trash-directory)
+            (progn
+              (when (file-directory-p dir-to-delete)
+                (delete-directory dir-to-delete))
+              ;; create dir-to-delete and delete it
+              (make-directory (expand-file-name dir-to-delete testdir))
+              (dired testdir)
+              (dired-toggle-marks)
+              (should (= 2 (length (directory-files trash-directory)))) ;; . and .. folders are there
+              (should (= 1 (length (dired-get-marked-files))))
+              (dired-do-delete nil)
+              (dired-toggle-marks)
+              (should (= 0 (length (dired-get-marked-files))))
+              (should (= 3 (length (directory-files trash-directory))))
+
+              ;; create dir-to-delete and delete it again
+              (make-directory (expand-file-name dir-to-delete testdir))
+              (revert-buffer)
+              (dired-toggle-marks)
+              (should (= 1 (length (dired-get-marked-files))))
+              (dired-do-delete nil)
+              (dired-toggle-marks)
+              (should (= 0 (length (dired-get-marked-files))))
+              (should (= 4 (length (directory-files trash-directory)))))))
+      (mapc (lambda (d)
+              (when (file-directory-p d)
+                (delete-directory d t)))
+            (list trash-directory testdir)))))
+
 (provide 'dired-tests)
 ;;; dired-tests.el ends here
-- 
2.44.0

